#!/bin/ash

. /lib/functions.sh
. /usr/share/libubox/jshn.sh

ucidef_add_nand_info() {
	local model="$1"

	model=${model:0:7}

	json_select_object nand

	case "$model" in
	TRB1412)
		json_add_int blocksize 128
		json_add_int pagesize 2048
		json_add_int subpagesize 2048
	;;
	TRB1422 |\
	TRB1423 |\
	TRB1424 |\
	TRB1452)
		json_add_int blocksize 256
		json_add_int pagesize 4096
		json_add_int subpagesize 4096
	;;
	TRB1420)
		json_add_int blocksize 256
		json_add_int pagesize 4096
		json_add_int subpagesize 4096
	;;
	TRB14*0)
		json_add_int blocksize 128
		json_add_int pagesize 2048
		json_add_int subpagesize 2048
	;;
	TRB14*1)
		json_add_int blocksize 256
		json_add_int pagesize 4096
		json_add_int subpagesize 4096
	;;
	esac

	json_select ..
}

ucidef_add_rmnet_proto_info() {
	local model="$1"

	case "$model" in
	TRB14*)
		json_add_string custom_proto "connm"
		json_add_string custom_ifname "rmnet0"
		;;
	esac
}

ucidef_add_custom_modem_info() {
	print_array() {
		json_add_array $1

		for element in $2; do
			json_add_string "" "$(echo $element)"
		done

		json_close_array
	}

	#Parameters: model primary
	local model vendor product boudrate gps type desc control region modem_counter
	modem_counter=1
	json_select_array "modems"

	model="$1"
	model=${model:0:7}

	case "$model" in
	TRB1422)
		vendor=05c6
		product=9215
		;;
	TRB1412 |\
	TRB1423 |\
	TRB1452 |\
	TRB140* |\
	TRB143*)
		vendor=2c7c
		product=0125
		;;
	TRB14*)
		vendor=2c7c
		product=0121
		;;
	esac

	case "$model" in
	TRB1412 |\
	TRB14*0)
		region="EU"
		[ "$product" = "0121" ] && {
			lte_bands="1 3 7 8 20 28"
			trysg_bands="wcdma_2100 wcdma_900"
			dug_bands="gsm_1800 gsm_900"
		}
		[ "$product" = "0125" ] && {
			lte_bands="1 3 7 8 20 28 38 40 41"
			trysg_bands="wcdma_2100 wcdma_900"
			dug_bands="gsm_1800 gsm_900"
		}
		;;
	TRB1422)
		region="CE"
		lte_bands="1 3 5 8 34 38 39 40 41"
		trysg_bands="bc-0-a-system bc-0-b-system wcdma_2100 wcdma_900"
		dug_bands="gsm_1800 gsm_900"
		;;
	TRB1423 |\
	TRB1452 |\
	TRB14*1)
		region="AU"
		[ "$product" = "0121" ] && {
			lte_bands="1 2 3 4 5 7 8 28 40"
			trysg_bands="wcdma_2100 wcdma_1900 wcdma_900 wcdma_850"
			dug_bands="gsm_1800 gsm_900 gsm_850 gsm_1900"
		}
		[ "$product" = "0125" ] && {
			lte_bands="1 2 3 4 5 7 8 28 40"
			trysg_bands="wcdma_2100 wcdma_1900 wcdma_900 wcdma_850"
			dug_bands="gsm_1800 gsm_900 gsm_850 gsm_1900"
		}
		;;
	esac

	[ -f "/lib/network/wwan/$vendor:$product" ] && {
		devicename="3-1"
		json_set_namespace defaults old_cb
		json_load "$(cat /lib/network/wwan/$vendor:$product)"
		json_get_vars gps boudrate type desc control stop_bits
		json_set_namespace $old_cb

		[ "${devicename%%:*}" = "$devicename" ] && {
			json_add_object
				json_add_string id "$devicename"
				json_add_string num "$modem_counter"
				json_add_string vendor "$vendor"
				json_add_string product "$product"
				json_add_string stop_bits "$stop_bits"
				json_add_string gps "$gps"
				json_add_string boudrate "$boudrate"
				json_add_string type "$type"
				json_add_string desc "$desc"
				json_add_string region "$region"
				json_add_string control "$control"
				json_add_int simcount 1
				json_add_boolean builtin 1
				json_add_string cmd_port "/dev/smd9"
				json_add_boolean primary 1
				json_add_object service_modes
					[ -z "$lte_bands" ] || print_array "4G" "$lte_bands"
					[ -z "$trysg_bands" ] || print_array "3G" "$trysg_bands"
					[ -z "$dug_bands" ] || print_array "2G" "$dug_bands"
				json_close_object
			json_close_object
		}
	}

	json_select ..
}

ucidef_target_defaults() {
	local model="$1"

	ucidef_add_nand_info "$model"
	ucidef_add_rmnet_proto_info "$model"
	ucidef_add_custom_modem_info "$model"
}

boot_target_pre_board_detect() {
	# /log directory might be created on preinit
	# symlink /storage to /log on TRB14X devices
	[ -d /storage ] && [ ! -h /log ] && {
		rm -rf /log
		ln -sf /storage /log
	}


	local param="$(/sbin/mnf_info --name)"
	[ "${param:0:6}" = "TRB141" ] && rm /etc/modules.d/spi-dev
}
